//=============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCDIOFunctions.c   v5.9
// Desc: Sample program that demonstrates how to use the digital I/O functions
// Created by	KL
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main GMMDSCDIOFunctions
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif


// var declarations
BYTE result;			// returned error code
DSCB dscb;		    	// handle used to refer to the board
DSCCB dsccb;			// structure containing board settings
BYTE portOut;			// port used for digital I/O
BYTE portIn;			// port used for digital I/O
BYTE output_b;			// BYTE sent to digital output
BYTE input_b;			// BYTE receiving digital input
BYTE config_bytes[2];	// configuration for input/output mode for dio ports
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variables of size int

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. DIO Input and Output
//	    IV. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//	  Initialize the driver, using the driver version for validation
	//
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the GMM board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf( "\nGMM BOARD INITIALIZATION:\n" );

	printf("Enter the base address (default 0x300) : ");
	scanf("%hx", &dsccb.base_address );

	printf("Enter interrupt1 level (2-7) : ");
	scanf("%d", &intBuff);
	dsccb.int_level1 = (BYTE) intBuff;

	printf("Enter interrupt2 level (2-7) : ");
	scanf("%d", &intBuff);
	dsccb.int_level2 = (BYTE) intBuff;

	if(dscInitBoard(DSC_GMM, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. DIO Setup
	//
	// The sample code below sets up the dio ports for input or output direction.
	// ports 0-2 are all set to input, ports 3-5 are all set to output. The sample
	// code below then writes to port 3 and through a cable, loops the signal to 
	// port 0. The signal is then read.
	//
	//=========================================================================

	printf("Enter the DIO configuration in hex for ports (0-2) : ");
	scanf("%x", &intBuff);
	config_bytes[0] = (BYTE)intBuff;	// set all ports on first chip to input

	printf("Enter the DIO configuration in hex for ports (3-5) : ");
	scanf("%x", &intBuff);
	config_bytes[1] = (BYTE)intBuff;		// set all ports on second chip to output

	dscDIOSetConfig(dscb, config_bytes);

   do{
		printf("Enter the port you want to output to : ");
		scanf("%d", &intBuff);
		portOut = intBuff;

		printf("Enter the code you want to output : ");
		scanf("%d", &intBuff);
		output_b = (BYTE) intBuff;

		printf("Enter the port you want to input from : ");
		scanf("%d", &intBuff);
		portIn = intBuff;

		getchar();

		if( dscDIOOutputByte(dscb, portOut, output_b) != DE_NONE)     // generating signal here and using cables to
		{														// loop back to #1 port C.
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscDIOutputByte error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}


		if( dscDIOInputByte(dscb, portIn, &input_b)	!= DE_NONE) // toggling.
		{
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscDIInputByte error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		}

		printf("Output %d to Output port %d; Input %d from Input port %d\n", (int)output_b, (int) portOut, (int)input_b, (int)portIn);
		printf("Press any q to exit or any key to continue.\n");
   } while( getchar() != 'q');


	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf( "\nDSCDIOFunctions completed.\n" );

	
	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
