//=============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCUserInt.c   v5.9 
// Desc: Sample program that demonstrates how to use the User Interrupt Function
// Created by	KL
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main GMMDSCUserInt
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif

// var declarations
BYTE result;        // returned error code
DSCB dscb;          // handle used to refer to the board
DSCCB dsccb;        // structure containing board settings
BYTE portOut;       // port used for digital I/O output
BYTE portIn;        // port used for digital I/O intput
BYTE output_b;      // BYTE sent to digital output
BYTE input_b;       // BYTE receiving digital input
BYTE config_bytes[2];	// digital I/O ports mode configuration
DSCUSERINT	dscuserint;

int counter;			// the global variable to be modified by the user interrupt function
void inc(void * param); // the user interrupt function

ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;           // temp variables of size int

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. User interrupt setup
//	    IV. Trigger the interrupts (just for demo purposes)
//	     V. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//	  Initialize the driver, using the driver version for validation
	//
	////=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the GMM board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf( "\nGMM BOARD INITIALIZATION:\n" );

	printf("Enter the base address (default 0x300) : ");
	scanf("%hx", &dsccb.base_address );

	printf("Enter interrupt1 level (2-7) : ");
	scanf("%d", &intBuff);
	dsccb.int_level1 = (BYTE) intBuff;

	printf("Enter interrupt2 level (2-7) : ");
	scanf("%d", &intBuff);
	dsccb.int_level2 = (BYTE) intBuff;

	getchar();

	if(dscInitBoard(DSC_GMM, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	// III. USER INTERRUPT SETUP
	//
	//	Sets up the following parameters for the user interrupt:
	//			user interrupt function
	//			interrupt source
	//			clock source (if the interrupt source is a counter/timer)
	//			interrupt rate (if the interrupt source is a counter/timer
	//	 				    and the clock source is internal)
	//
	//	The sample code below sets dsccb.int_level1 (interrupt 1) and sets the
	//	mode to C0. Its then using 82c55 #2 port C to driver the pins for
	//	82c55 #1 port C. User can use anything to drive or generate signal/voltage
	//	change for the #1 port C. 
	//=========================================================================

	counter = 0;				// set the counter to 0

	dscuserint.rate = 0;		// N/R (not required) No other option.
	dscuserint.counter = 0;		// N/R. No other option.
	dscuserint.intsource = 0;  // external source. There is no internal clock;
	dscuserint.clksource = 0;  // N/R. No other option.
	
	config_bytes[0] = 0x9b;		// set all ports on first chip to input	
	config_bytes[1] = 0x80;		// set all ports on second chip to output

	dscDIOSetConfig(dscb, config_bytes);		// setup the dio port configurations 
												// for input or output

	if( dscUserInt(dscb, &dscuserint, inc) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscUserInt error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//===========================================================================
	//	IV. CASES FOR USERINT 
	//
	//      Tests userinterrupt
	//===========================================================================

	counter = 0;		// counts number of times userint has been called

	do
	{
		if( (result = dscDIOOutputByte(dscb, 5, 0)) != DE_NONE)     // generating signal here and using cables to 
		{															// loop back to #0 port #5.
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscDIOOutputbyte error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
			return 0;
		}
											
		if( (result = dscDIOOutputByte(dscb, 5, 0x01)) != DE_NONE)	//toggling port 5
		{															
			dscGetLastError(&errorParams);
			fprintf( stderr, "dscDIOOutputByte error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
			return 0;
		}
					
		printf("Counter %d\n", counter);

		printf("Press 'q' to quit or ENTER to continue\n");
		
	}while(getchar() != 'q');

	// cancel interrupts manually if interrupts are still running
	if( (result = dscCancelOp(dscb)) != DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscCancelOp error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	if( (result = dscClearUserInterruptFunction(dscb)) != DE_NONE)  // clears interrupts.
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscClearUserInterruptFunction error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	//=========================================================================
	//  V. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf( "\nDSCDIOFunctions completed.\n" );

	return 0;
} // end main()

//=============================================================================
//	The user interrupt routine
//
//	Note: If using DOS, you cannot execute dsc driver functions to access the board
//	both in the main program and in the user interrupt routine. This is because
//	the driver locks the board in the main program when it is being used so that
//	no other program can access it and interfere with a function while it is executing.
//	To access the board inside the interrupt routine in DOS, use direct I/O functions
//	( inp(addr), outp(addr,data) ).
//
//=============================================================================

void inc(void * param)
{
	counter ++;
	if( (result = dscDIOInputByte(dscb, 2, &input_b)) != DE_NONE)  //must access board to clear interrupt		 
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscDIOInputByte error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
	}
}


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
